/*###############################################################################
## ETiles project
##
## Copyright (C) 2010 Carlos Villegas Núñez  carlos.villegasnunez@alum.uca.es
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###############################################################################*/

#include "maleatorios.h"
#include "ui_maleatorios.h"

maleatorios::maleatorios(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::maleatorios)
{
    ui->setupUi(this);

    //Enlazamos la interfaz de lista junto con su modelo de datos
    tiles = new Tile(this,&listatiles);
    listatiles.setModel(tiles);

    //Añadimos al contenedor vertical la lista de tiles junto con su barra de herramientas
    ui->cvertical->addWidget(&listatiles);
    ui->cvertical->addWidget(listatiles.BarraAcciones());

    ui->lsemilla->setText(QString::number(ui->ssemilla->value()));

    //Añadimos los diferentes algoritmos que pueden seleccionarse para la creación de mapas aleatorio
    ui->algsel->addItem(QString("Algoritmo Perlin Noise"));
    //ui->algsel->addItem(QString("Algoritmo Gausiano"));

    //Conectamos las acciones de la barra de herramientas junto con sus funciones
    connect(listatiles.AccionEliminar(),SIGNAL(triggered()),this,SLOT(EliminarTile()));
    connect(listatiles.AccionAnadir(),SIGNAL(triggered()),this,SLOT(AnadirTile()));
    connect(listatiles.AccionTileset(),SIGNAL(triggered()),this,SLOT(AnadirTileSet()));
}

maleatorios::~maleatorios()
{
    delete ui;
}

void maleatorios::changeEvent(QEvent *e)
{
    QDialog::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void maleatorios::on_buttonBox_accepted()
{
    QList<QPixmap> ltiles;
    QList<QString> lnombres;
    //Insertamos los tiles y rutas de cada uno de ellos en lista auxiliares que se copiarán a la lista de tiles del mapa.
    for(int i=0; i<tiles->Tamano();i++)
    {
        ltiles.push_back(tiles->ImagenTile(i));
        lnombres.push_back(tiles->NombreTile(i));
    }
    ((VentanaPrincipal*)(this->parentWidget()))->CrearMapaAleatorio(ui->EAnchuraTile->text().toInt(),
                                                                    ui->EAlturaTile->text().toInt(),
                                                                    ui->ENumAnch->text().toInt(),
                                                                    ui->ENumAltu->text().toInt(),
                                                                    ltiles,lnombres,
                                                                    ui->saguatierra->value(),
                                                                    ui->scontislas->value(),
                                                                    ui->smonvar->value(),
                                                                    ui->ssemilla->value(),
                                                                    ui->algsel->currentIndex());
    accept();
}

void maleatorios::on_buttonBox_rejected()
{
    reject();
}

void maleatorios::AnadirTile()
{
    QPixmap newImage;
    QString fileName = QFileDialog::getOpenFileName(this);
    if(!fileName.isEmpty())
        if (!newImage.load(fileName))
        {
                QMessageBox::warning(this, tr("Importar Tile"),tr("La imagen no puede ser cargada"),QMessageBox::Cancel);
        }
        else
        {
            tiles->addPiece(newImage, fileName);
        }
}

void maleatorios::EliminarTile()
{
    int i=listatiles.TileSeleccionado();

    if (i!= -1)
    {
        tiles->removeRow(i);
    }
}

void maleatorios::AnadirTileSet()
{
    cargartileset ventanac(this,tiles,ui->EAnchuraTile->text().toInt(),ui->EAlturaTile->text().toInt());
    ventanac.exec();
}

void maleatorios::on_ssemilla_valueChanged(int value)
{
    ui->lsemilla->setText(QString::number(ui->ssemilla->value()));
}
