/*###############################################################################
## ETiles project
##
## Copyright (C) 2010 Carlos Villegas Núñez  carlos.villegasnunez@alum.uca.es
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###############################################################################*/

#ifndef TILE_H
 #define TILE_H

 #include <QAbstractListModel>
 #include <QList>
 #include <QPixmap>
 #include <QPoint>
 #include <QStringList>
 #include "listatiles.h"

 class QMimeData;
 class ListaTiles;

 class Tile : public QAbstractListModel
 {
     Q_OBJECT

 public:
     /** Constructor predeterminado de la clase tile, inicializa los datos para guardar los tiles de la lista de tiles.
      * @param parent: Puntero al padre que llama al constructor
      */
     Tile(QObject *parent = 0);
     /** Constructor predeterminado de la clase tile, inicializa los datos para guardar los tiles de la lista de tiles.
      * @param parent: Puntero al padre que llama al constructor.
      * @param lista: Interfaz de los datos de la lista de tiles.
      */
     Tile(QObject *parent, ListaTiles *lista);
     /** Destructor de la clase tile*/
     ~Tile();

     /** Método constante que devuelve la forma de representar los datos dentro del modelo a la hora de producirse un evento Drag sobre ella
      * @param index: Índice del dato dentro de la lista
      * @param role: Rol sobre el cual se devolverá la representación del dato
     */
     QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;     
     /** Método constante que devuelve el número de tiles de la lista
      * @param index: Indica el padre del modelo de la lista
     */
     int rowCount(const QModelIndex &parent = QModelIndex()) const { return tiles->size();}
     /** Procedimiento que añade un elemento a la lista de tiles
       * @param pixmap: Imagen del tile que representa
       * @param nombret: Nombre del fichero que contiene la imagen del tile
       * @param sep: Separación entre cada tile.
       */
     void addPiece(const QPixmap &pixmap=QPixmap(), const QString nombret=QString(), int sep =0);
     /** Método que elimina un dato de la lista, y devuelve si se ha producido correctamente
       * @param row: Indice del dato que se va a remover
       * @param parent: Indica el padre del modelo  de la lista
       */
     bool removeRow(int row, const QModelIndex &parent = QModelIndex());
     /** Método que devuelve si se ha producido alguna modificación desde la última vez que se guardó la lista*/
     int ModificadoDesdeGuardado() const { return modificado;}
     /** Procedimiento que se llama cada que vez que se guarda la lista */
     void TileGuardado() { modificado =0;}
     /** Método que devuelve el tamaño de la lista*/
     int Tamano() const { return tiles->size();}
     /** Método que devuelve el número de letras del nombre de un tile
      * @param i: Índice del nombre que queremos acceder
      */
     int Cantidad(int i) const { return nombres.count(nombres.at(i));}
     /** Método que devuelve la posición del primer tile que forma parte del tileset que se le pasa como parámetro
      * @param i: Posición dentro de la lista
      */
     int PrimeraPosTileSet(int i) { return nombres.indexOf(nombres.at(i));}
     /** Método que devuelve la posición del último tile que forma parte del tileset que se le pasa como parámetro
      * @param i: Posición dentro de la lista
      */
     int UltimaPosTileSet(int i) { return nombres.lastIndexOf(nombres.at(i));}
     /** Método que devuelve la imagen del tile correspondiente al tile indicado
      * @param Índice del tile
      */
     QPixmap ImagenTile(int i) const { return tiles->at(i);}
     /** Método que devuelve la separación del tile correspondiente al tile indicado
      * @param Índice del tile
      */
     int SeparacionTile(int i) const { return separaciones.at(i);}
     /** Método que devuelve el nombre del tile correspondiente al tile indicado
      * @param Índice del tile
      */
     QString NombreTile(int i) const { return nombres.at(i);}
     /** Método constante que devuelve las banderas del modelo de datos
      * @param index: Indica el padre del modelo de la lista
     */
     Qt::ItemFlags flags(const QModelIndex &index) const;
     /** Método que devuelve la lista de imagenes de los tiles*/
     QList<QPixmap>* ListaImagenes() const {return tiles;}
     ListaTiles *vistalistatiles;
 private:
     int modificado;
     QList<QPixmap> *tiles;
     QList<QString> nombres;
     QList<int> separaciones;

 };

 #endif
